/* ImportSymbols.rexx - Script which is adding symbols into ACE from:
 *   - a file containing a list of EQU  (source files)
 *   - a listing file generated by vasm (lst files)
 *   - a symbols file generated by rasm (sym files, not rasm files!)
 *
 * Syntax: ImportSymbols.rexx <file name> vasm|rasm|equ [undefined|ram-main|ram-extension|rom-extension|rom-firmware|rom-cartridge|ram-asic [<map bank> [<map page>]]]
 *
 * $VER: ImportSymbols.rexx 1.4 (15.12.23)  2017-2023 by Philippe 'OffseT' Rimauro <offset@cpcscene.net>
 * Requires ACE 1.26 or later
 */

OPTIONS RESULTS
SIGNAL ON BREAK_C

ace_required_version="1.26"
ace_port="ACE.1"

appname="ACE Rexx"
LF="0A"x

PARSE ARG symbol_file file_type map_type bank page

/* Request main arguments if not provided */
temp="T:symb.req"
pattern="#?"

IF file_type="" THEN DO
    ADDRESS 'COMMAND' 'RequestChoice >'temp' TITLE "'||appname||'" BODY "Please select which type of import should be done." GADGETS "EQU|vasm|rasm|Cancel"'
    IF OPEN(handle,temp,"R")~=0 THEN DO
        RESULT=READLN(handle)
        IF RESULT=0 THEN EXIT
        IF RESULT=1 THEN DO
            file_type="equ"
            pattern="#?.(asm|a|i|z80)"
        END
        ELSE IF RESULT=2 THEN DO
            file_type="vasm"
            pattern="#?.lst"
        END
        ELSE IF RESULT=3 THEN DO
            file_type="rasm"
            pattern="#?.sym"
        END
        CLOSE(handle)
    END
END

IF symbol_file="" THEN DO
    ADDRESS 'COMMAND' 'RequestFile >'temp' TITLE "Please choose the symbol file to import" PATTERN '||pattern
    IF RC~=0 THEN EXIT FatalError("No file selected!","Gasp!")
    IF OPEN(handle,temp,"R")~=0 THEN DO
        RESULT=READLN(handle)
        PARSE VAR RESULT '"'symbol_file'"'
        CLOSE(handle)
    END
END

/* Set other arguments to defaults */
IF map_type="" THEN DO
    map_type="Undefined"
    END

IF bank="" THEN DO
    bank=0
    END

IF page="" THEN DO
    page=0
    END

/* Check arguments */
IF symbol_file="" | file_type="" THEN DO
    EXIT FatalError("Error in arguments!","Gasp!")
END
IF file_type~="vasm" & file_type~="rasm" & file_type~="equ" THEN DO
    EXIT FatalError("Only file types equ, vasm and rasm are supported!","Oops!")
END

/* Look for ACE running */
IF ~SHOW("P",ace_port) THEN DO
    EXIT FatalError("ACE CPC Emulator process not found!","Damned!")
END

/* ACE is not the default port */
ADDRESS(ace_port)

/* Check ACE version */
"VERSION"
IF CheckVersion(RESULT,ace_required_version)~=0 THEN DO
    EXIT FatalError("ACE CPC Emulator version" ace_required_version "or better is required!","I will update!")
END

/* Load required external libraries */
IF LoadLib("rexxsupport.library")~=0 THEN DO
    EXIT FatalError("rexxsupport.library not found!*NPlease install it.","Okay!")
END

/* Pause ACE while working */
"PAUSE"

/* Add symbols */
retval = 20;

IF file_type="vasm" THEN DO
    retval = AddSymbols_vasm(symbol_file,map_type,bank,page)
    END
ELSE IF file_type="rasm" THEN DO
    retval = AddSymbols_rasm(symbol_file,map_type,bank,page)
END
ELSE IF file_type="equ" THEN DO
    retval = AddSymbols_equ(symbol_file,map_type,bank,page)
END

IF retval~=0 THEN SAY "Symbols file '"symbol_file"' could not be added!"

/* Start ACE again */
"START"

EXIT

/*
 * Add symbols vasm (providing a listing file generated by vasm)
 */
AddSymbols_vasm:
    PARSE ARG file_name,map_type,bank,page

    IF OPEN(file_handle,file_name,"R")~=0 THEN DO
        DO WHILE ~EOF(file_handle)
            line=READLN(file_handle)
            IF line="Symbols:" THEN DO WHILE ~EOF(file_handle)
                line=READLN(file_handle)
                PARSE UPPER VAR line label " EXPR(" dec_addr "=" hex_addr ")"type
                IF label~="" & dec_addr~="" & hex_addr~="" & type~="" & type~="INTERNAL" THEN DO
                    AddSymbol(label, hex_addr, map_type, bank, page)
                END
            END
        END
        CLOSE(file_handle)
    END
    ELSE RETURN 20

    RETURN 0

/*
 * Add symbols rasm (providing a symbol file generated by rasm)
 */
AddSymbols_rasm:
    PARSE ARG file_name,map_type,bank,page

    IF OPEN(file_handle,file_name,"R")~=0 THEN DO
        DO WHILE ~EOF(file_handle)
            line=READLN(file_handle)
            PARSE VAR line label "#"hex_addr unused sym_type
            IF label~="" & hex_addr~="" & unused~="" THEN DO
                IF sym_type="L" THEN DO
                    AddSymbol(label, "0x"hex_addr, map_type, bank, page, "")
                END
                ELSE IF sym_type="I" THEN DO
                    AddSymbol(label, "0x"hex_addr, "Undefined", 0, 0, "RELATIVE")
                END
                ELSE DO
                    AddSymbol(label, "0x"hex_addr, "Undefined", 0, 0, "")
                END
            END
        END
        CLOSE(file_handle)
    END
    ELSE RETURN 20

    RETURN 0

/*
 * Add symbols equ (providing a file containing a list of EQU; suitable for pasmo)
 */
AddSymbols_equ:
    PARSE ARG file_name,map_type,bank,page

    IF OPEN(file_handle,file_name,"R")~=0 THEN DO
        DO WHILE ~EOF(file_handle)
            line=READLN(file_handle)
            PARSE UPPER VAR line label "EQU" "&"hex_addr
            IF label="" THEN DO
                PARSE UPPER VAR line label "EQU" "#"hex_addr
            END
            IF hex_addr="" THEN DO
                PARSE UPPER VAR line label "EQU" "0"hex_addr"H"
            END
            IF hex_addr="" THEN DO
                PARSE UPPER VAR line label "EQU" dec_addr
            END
            IF label~="" THEN DO
                IF hex_addr~="" THEN DO
                    AddSymbol(label, "0x"hex_addr, map_type, bank, page)
                END
                ELSE IF dec_addr~="" THEN DO
                    AddSymbol(label, hex_addr, map_type, bank, page)
                END
            END
        END
        CLOSE(file_handle)
    END
    ELSE RETURN 20

    RETURN 0

/*
 * Add one symbol into ACE
 */
AddSymbol:
    PARSE ARG label,addr,mt,mb,mp,more

    "ADDSYMBOL NAME" label "ADDRESS" addr "MAPTYPE" mt "BANK" mb "PAGE" mp more
    IF rc~=0 THEN DO
        SAY "Error symbol:" label "=" addr
    END

    RETURN 0

/*
 * Version checker (ver.rev[beta])
 */
CheckVersion:
    PARSE ARG cv "." cr "beta" , rv "." rr "beta"

    IF cv<rv | (cv=rv & cr<rr) THEN DO
        RETURN 5
    END
    RETURN 0

/*
 * External lib loader
 */
LoadLib:
    PARSE ARG lib

    IF SHOW("L",lib)=0 THEN DO
        IF ADDLIB(lib,0,-30,0)=0 THEN DO
            RETURN 20
        END
    END
    RETURN 0

/*
 * Fatal error
 */
FatalError:
    PARSE ARG message,button

    ADDRESS "COMMAND" 'RequestChoice >NIL: TITLE "'||appname||'" BODY "'||message||'" GADGETS "'||button||'"'
    RETURN 20
